;*****************************************************************************
;                                                                            *
;    NOM               : Thermostat                                          *
;    Date cration     : Mars 2011                                           *
;    Date modification :                                                     *
;    Version           : 1                                                   *
;    Circuit           : PCB2011/3                                           *
;    Auteur            : Laurent Counotte                                    *
;                                                                            *
;*****************************************************************************
;                                                                            *
;    Fichier requis:                                                         *
;                    affichag.inc  											 *
;					 binDec.inc  											 *
;					 LCD_rout.inc 											 *
;					 Message.inc											 *
;			   		 onewire.inc                            			     *
;					 														 *
;                    														 *
;                                                                            *
;    Frquence de travail :  4 MHz                                           *
;    Frquence du quartz  :  4 MHz                                           *
;                                                                            *
;*****************************************************************************
; Ce programme affiche une temprature sur un LCD. Le capteur est un DALLAS  *
; DS1820 qui ncessite d'implmenter une routine de gestion du BUS one-wire  *
; (un fil). Une fois rcupr l'octet final  afficher on le convertit 	     *
; en un chiffre dcimal (centaine, dizaine units) en utilisant la routine   *
; "BINDEC.INC". Enfin, on affiche grace  la  routine "AFFICHAGE.INC" des    *
; messages (MESSAG.INC) les lments du rsultat en utilisant la 			 *
; routine "LCD_ROUTINES.INC".      											 *
; ****************************************************************************
;
;
;Pins utilises :                                                            *
;  ----------------                                                          *
;                                                                            *
;    RA0/AN0/VRef     :  LM35                                                *
;    RA1/AN1          :  H25K5A                                              *
;    RA2/AN2/Vref-    :  -                                                   *
;    RA3/AN3/Vref+    :  -                                                   *
;    RA4/T0CKI        :  -                                                   *
;    RA5/AN4/SS/LVDIN :  DS18B20                                             *
;                                                                            *
;    OSC1/CLKI        :  OSC                                                 *
;    OSC2/CLKO/RA6    :  OSC                                                 *
;   																		 *
;    RB0/INT0         :  DATALCD4                                            *
;    RB1/INT1         :  DATALCD5                                            *
;    RB2/CANTX/INT2   :  DATALCD6                                            *
;    RB3/CANRX        :  DATALCD7                                            *
;    RB4              :  RSLCD                                               *
;    RB5              :  ENLCD                                               *
;    RB6              :  COMMANDE                                            *
;    RB7              :  E (RS485)  										 *
;                                                                            *
;    RC0/T1OSO/T1CKI  :  LED1                                                *
;    RC1/T1OSI        :  LED2                                                *
;    RC2/CCP1         :  LED3                                                *
;    RC3/SCK/SCL      :  BP1                                                 *
;    RC4/SDI/SDA      :  BP2                                                 *
;    RC5/SDO          :  BP3                                                 *
;    RC6/TX/CK        :  TX (RS485)                                          *
;    RC7/RX/DT        :  RX (RS485)                                          *
;                                                                            *

;Fichiers utilises :                                                            
;---------------- 

	LIST		p=18F2420            		; Dfinition de processeur
    #include	<P18F2420.inc>        		; fichier include

;*****************************************************************************
;                          CONFIGURATIONS                                    *
;*****************************************************************************

 CONFIG	OSC = HS		; Oscillateur principal (LP,XT,HS,RC,EC,ECIO,HSPLL,RCIO)
 
 IFNDEF DEBUG
 CONFIG DEBUG = OFF		; Debugger hors service (laisser sur OFF)
 CONFIG FCMEN = OFF
 CONFIG IESO = OFF
 CONFIG PWRT = ON		; Dlai au dmarrage (ON/OFF)
 CONFIG BOREN = OFF		; Reset si chute de tension (ON/OFF)
 CONFIG BORV = 0		; 0,1,2 ou 3 (settings de la tension)
 CONFIG WDT = ON		; Mise en service watchdog (ON/OFF)
 CONFIG WDTPS = 1		; Postdiviseur du watchdog (1,2,4,8,16,32,64,128)
 CONFIG CCP2MX = PORTB    ; Multiplexage CCP
 CONFIG PBADEN = OFF    ; A/N PortB dsactiv
 CONFIG LPT1OSC = OFF 
 CONFIG MCLRE = ON		; Pin MCLR activ (5v)
 CONFIG STVREN = ON		; Reset sur dbordement de pile (ON/OFF)
 CONFIG LVP = OFF		; Programmation basse tension autorise (ON/OFF)
 CONFIG XINST = OFF		; Programmation basse tension autorise (ON/OFF)
 CONFIG CP0 = OFF		; code protection sur block 0 (ON/OFF)
 CONFIG CP1 = OFF		; code protection sur block 1 (ON/OFF)
 CONFIG CPB = OFF		; code protection sur bootblock  (ON/OFF)
 CONFIG CPD = OFF		; Code protection sur eeprom (ON/OFF)
 CONFIG WRT0 = OFF		; Protection criture block 0 (ON/OFF)
 CONFIG WRT1 = OFF		; Protection criture block 1 (ON/OFF)
 CONFIG WRTC = OFF		; Protection criture configurations (ON/OFF)
 CONFIG WRTB = OFF		; Protection criture bootblock (ON/OFF)
 CONFIG WRTD = OFF		; Protection criture zone eeprom (ON/OFF)
 CONFIG EBTR0 = OFF		; protect. lecture de table block 0 (ON/OFF)
 CONFIG EBTR1 = OFF		; protect. lecture de table block 1 (ON/OFF)
 CONFIG EBTRB = OFF		; protect. lecture de table bootblock (ON/OFF)
 ENDIF  

 
;=============================================================================
;                               ZONE EEPROM                                  =
;=============================================================================
	ORG 0xF00000

;*********************************************************************
;                             DEFINE                                 *
;*********************************************************************

; PORT A
; ---------

#DEFINE LM35			PORTA,0			; Capteur LM35 (AN) sur RA0
#DEFINE H25K5A			PORTA,1			; Capteur H25K5A (AN) sur RA1
#DEFINE BUS_DS18S20		PORTA,5			; Capteur DS1820 "one-wire" sur RA4


; PORT B
; ---------

#DEFINE DATA_LCD4		PORTB,0			; DATA4 LCD sur RB0
#DEFINE DATA_LCD5		PORTB,1			; DATA5 LCD sur RB1
#DEFINE DATA_LCD6		PORTB,2			; DATA6 LCD sur RB2
#DEFINE DATA_LCD7		PORTB,3			; DATA7 LCD sur RB3
#DEFINE LCD_RS			PORTB,4			; Inhibe RS sur RB4			
#DEFINE	LCD_E			PORTB,5    		; Active signal E sur RB5
#DEFINE COMMANDE		PORTB,6			; Commande sortie vers relais sur RB6
#DEFINE ENABLE_RS485	PORTB,7			; Autorise la com RS485 sur RB7
											

; PORT C
; ---------

#DEFINE LED_ROUGE		PORTC,0			; LED de sortie 1 (Rouge - max)
#DEFINE LED_VERT		PORTC,1			; LED de sortie 2 (Vert - OK)
#DEFINE LED_BLEU		PORTC,2			; LED de sortie 3 (Bleu - min)
#DEFINE BP_MODE			PORTC,3			; BP1 fonction MODE
#DEFINE BP_UP			PORTC,4			; BP2 fonction UP
#DEFINE BP_DOWN			PORTC,5			; BP3 fonction DOWN
#DEFINE TX_485			PORTC,6			; TX RS485
#DEFINE RX_485			PORTC,7			; RX RS485

;*****************************************************************************
;                         VARIABLES ACCESS RAM                               *
;*****************************************************************************

; zone de 96 octets
; -----------------


 CBLOCK	0x00					; zone access ram
	
; variables pour les routines divOct et binDec
	dividende		: 1			; la valeur  afficher
	diviseur		: 1  		; 100 ou 10 (pour 3digits)
	resultat		: 1			; rsultat de la division
	reste			: 1			; reste de la division = units
	centaine		: 1			; nombre de centaines
	dizaine			: 1			; nombre de dizaines
	unites			: 1			; le reste
	decimale		: 1			; valeur de la dcimale
	
; variables pour la routine ONEWIRE
	compteur 		: 1			; compteur de bit de l'octet
	buffer			: 1			; buffer de stockage pour READ ou WRITE
	bufferL 		: 1			; stockage de la temprature aprs conversion
	bufferH			: 1			; dans variables haute et basse
	tempo			: 1			; variable utilise pour les micro-dlais
	temporaire		: 1			; variable de stockage temporaire de la valeur du BUS
	octet__afficher: 1			; octet rsultant de bufferL et bufferH  afficher

; variables pour l'afichage LCD (affichage.inc)
	Tempo_1_LCD		: 1
	Tempo_2_LCD		: 1
	Tempo_3_LCD		: 1

; variables pour l'afichage des messages (Message.inc)
	Tempo_1_MES		: 1
	Tempo_2_MES		: 1
	Tempo_3_MES		: 1

; variables pour le LCD
	LCDtmp			: 1
	LCDtmp2			: 1	
	LCDt1			: 1
	LCDt2			: 1
	LCDt3			: 1
	LCDt4			: 1	

; variables pour l'alarme		 
	seuilBas		: 1			; variable seuilBas
	seuilHaut		: 1			; variable seuilHaut
	
; variable d'affichage
	flag			: 1			; permet de choisir entre "Alarme1" et "Alarme2"	
						

 ENDC							; Fin de la zone                       


;*****************************************************************************
;                        VARIABLES BANQUE 0                                  *
;*****************************************************************************
; zone de 160 octets, suite de l'access RAM
; -----------------------------------------
 CBLOCK	0x60

 ENDC
;*****************************************************************************
;                        VARIABLES BANQUE 1                                  *
;*****************************************************************************
; zone de 256 octets
; ------------------
 CBLOCK 0x100
 
 ENDC

;*****************************************************************************
;                        VARIABLES BANQUE 2                                  *
;*****************************************************************************
; zone de 256 octets
; ------------------
 CBLOCK 0x200

	; attention, pour le 18F248, les adresses 0x2F4  0x2FF sont utilises
	; par l'ICD2

 ENDC

;*****************************************************************************
;                      DEMARRAGE SUR RESET                                   *
;*****************************************************************************

	ORG 0x0						; Adresse de dpart aprs reset		

	#include "LCD_rout.inc"		; routines de niveau 1 de gestion du LCD
	#include "affichag.inc"		; routines de niveau 2 d'affichage sur LCD
	#include "binDec.inc"		; routine de conversion binaire-dcimale
	#include "onewire.inc"		; routine de gestion du BUS one-wire
	#include "Message.inc"		; routine de gestion des messages affich

	goto    init				; Adresse 0

;*****************************************************************************
;                       INITIALISATIONS                                      *
;*****************************************************************************
init

	movlw	b'10000000'			; rglage registre INTCON2 - rsistance de rappel hors service (b7)
    movwf	INTCON2				; Charge le masque dans le registre Intcon2

	movlw	b'00000000'			; initialiser
	movwf	INTCON				; registre INTCON

; Configuration des PORTS (IN/OUT) - (ANALOGIQUE/DIGITAL)

	movlw	b'00001101'			; PORTA en mode digital AN0 & AN1 (voir tableau PIC page 224)
	movwf	ADCON1				; criture dans le registre contrle A/D (tableau PIC page 224)

	
	movlw	b'00100011'			; rglage du portA I/O - RA4, RA1, RA0 en entres (1=entre)
	movwf	TRISA

	clrf	TRISB				; PORTB en sorties

	movlw	b'01111000'			; rglage du portC I/O - RC6, RC5, RC4, RC3 en entres
	movwf	TRISC

; Mise  0 des PORTS

	clrf	PORTA				; Sorties portA  0
	clrf	PORTB				; Sorties portB  0
	clrf	PORTC				; Sorties portC  0

	bcf		LCD_RS				; passe le LCD en MODE COMMANDE (Inhibe RS)
	bcf		LCD_E				; LCD HORS SERVICE (Inhibe signal E)

; Configuration du DS18B20 en mode 9 bits -- TH, TL et CONF sont crits dans le
; SCRATCHPAD
	call	INIT				; squence d'init
	movlw	H'CC'				; ROM Command SKIP ROM = CCh
	movwf	buffer				; le code est charg dans le buffer
	call	WRITOCT				; transmission de la commande  l'esclave sur le BUS
	movlw	H'4E'				; Function Command -- write scratchpad = 4Eh
	movwf	buffer				; le code est charg dans le buffer
	call	WRITOCT				; transmission de la commande  l'esclave sur le BUS
	movlw	b'00000000'			; premier octet ; sans importance
	movwf	buffer				; le code est charg dans le buffer
	call	WRITOCT				; transmission de la commande  l'esclave sur le BUS
	movlw	b'00000000'			; second octet ; sans importance
	movwf	buffer				; le code est charg dans le buffer
	call	WRITOCT				; transmission de la commande  l'esclave sur le BUS
	movlw	B'00011111'			; troisime octet : configuration en mode 9 bits
	movwf	buffer				; le code est charg dans le buffer
	call	WRITOCT				; transmission de la commande  l'esclave sur le BUS

; Initialisation du LCD
	call	LCDinit				; squence d'initialisation du LCD
	movlw	b'00001100'			; LCD ON, Curseur OFF et blink OFF
	call	LCDins				; excute

; Initialisation des seuils
	movlw	d'25'				; 25C
	movwf	seuilHaut			; dans seuilHaut
	movlw	d'22'				; 22C
	movwf	seuilBas			; dans seuilBas
	bcf		flag,0				; initialise la variable "flag"
	bcf		BUS_DS18S20			; clr RA5
	bsf		COMMANDE			; clr RB6

	goto	main				; dmarrage


;*****************************************************************************
;                            PROGRAMME PRINCIPAL                             *
;*****************************************************************************

main							; Call(s) de la routine ONEWIRE.INC
	call	CONVERT_T			; on appelle une conversion de temprature
	call	LEC_TEMP			; on rapatrie les 2 octets 'bufferL' et 'bufferH'
	call	DECIMALE			; on traite la dcimale
	call	RANGER				; on traite les 2 octets pour n'en avoir plus qu'un
								; et on traite le signe

; ----------------------------------------------------------------------------
;  ce stade, on connat le signe de la temprature, on dispose de l'octet 
;qu'il va falloir traiter et on connat la dcimale on passe  l'affichage 
;si un seuil est atteint

AFFICHAGE
	call	afficher_temp	; on traite uniquement les valeurs positives
	
ALARME1	
	bcf		STATUS,C				; clearer le bit de carry
	movf	octet__afficher,w		; rappeler l'octet  afficher
	subwf	seuilHaut,w				; seuilHaut - octet__afficher
	btfsc	STATUS,C				; tester le bit de carry
	goto	main					; si C=1, on ne dpasse pas le seuil haut,
									; nouvelle mesure	
	call	MARCHE					; si C=0, afficher Message "Demarre ventilo"
	bcf		COMMANDE				; donner la sortie sur RA4 (attention drain ouvert)
	bsf		flag,0					; on positionne le flag poour passer sur Alarme2
	goto	main					; on boucle

ALARME2
	bcf		STATUS,C				; clearer le bit de carry
	movf	octet__afficher,w		; rappeler l'octet  afficher
	subwf	seuilBas,w				; seuilBas - octet__afficher
	btfss	STATUS,C				; tester le bit de carry
	goto	main					; si C=0, on boucle sur le programme -->nouvelle mesure	
	call	ARRET					; si C=1, afficher Message "Arret ventilo"
	bsf		COMMANDE				; si C=1, on coupe le ventilo
	bcf		flag,0					; on efface le flag pour revenir  Alarme1
	goto	main						; on redmarre 
	
fin
	bra		fin

	END						; fin de programme