;                               BinDec.INC
;**********************************************************************************
; Le DS18B20 de chez DALLAS est capable de retourner une temprature comprise entre 
; +125C et -55C avec une prcision de 4 dcimales et le signe, valeur qui nces-
; site deux octets. On se limitera volontairement  la prcision d'une dcimale ce
; qui permettra de travailler sur un seul octet pour la partie entire. On s'arran-
; gera pour que le premier bit (2^7) de l'octet corresponde au signe et les 7 bits 
; suivants  la partie entire de la temprature ; la partie dcimale sera directe-
; ment dduite du 9e bit s'il est gale  '0', la dcimale est nulle ; s'il vaut '1'
; la dcimale vaudra 0,5						
;
; Cette routine permet d'afficher une valeur dcimale (par exemple +042,5 ou -018,3)
; sur un afficheur LCD. La valeur disponible est l'octet binaire qui proviendra du
; capteur de temprature DS18B20. On le convertira pour pouvoir afficher le signe + 
; ou -, les centaines, les dizaines, les units, la virgule et une dcimale.
; Comme la division n'est videmment pas implmenter dans les contrleurs, il faut 
; la remplacer par autant de soustractions que ncessaires. La routine 'divOct' 
; soustrait le diviseur du dividende et teste le rsultat. Si le resultat est nul,  
; alors le dividende est un multiple du diviseur on teste le bit de carry qui n'est  
; donc pas positionn (=0). Le nouveau dividende = 0 et la suite est triviale. 
; Si le rsultat est <0, alors on teste aussi le bit de carry. Cette fois le carry 
; est positionn (=1). On est donc all trop loin, le reste est ngatif, il faut 
; revenir en arrire. On enlve 1 au resultat, on rajoute le diviseur  la dernire 
; valeur du dividende qu'on recopie dans le reste.
;
;*********************************************************************************

divOct							; routine utilise par la routine principale
	clrf	resultat			; initialisation de la variable 'resultat'
	clrf	reste				; initialisation de la variable 'reste'
	
aPasse							; on soustrait le diviseur du dividende autant de
	clrwdt						; fois qu'il est possible
	incf	resultat, F			; incrementer le resultat (on compte le nombre de
								; fois qu'on a soustrait le diviseur)
	movf	diviseur, W			; charger le diviseur dans w
	subwf	dividende, F		; soustraire (dividende - diviseur)
	btfss	STATUS,Z			; on teste le resultat ; s'il est nul 
								; alors Z=1 on excute la routine 'aPassePlus'
	btfss	STATUS, C			; sinon on teste le carry
	goto	aPassePlus			; si le carry=1 -> excuter la routine 'aPassePlus' 
	goto	aPasse				; si le carry=0 -> excuter la routine 'aPasse'				

aPassePlus						; on a vrifi qu'une soustraction supplmentaire 
	clrwdt						; ne passait pas
	btfsc	STATUS, C			; on teste le carry, si =1, le resultat est 0, 
	return						; on peut quitter
	decf	resultat,f			; et si on a carry =0, le resultat de la soustaction
								; est negatif, on decremente la variable 'rsultat' 
								; car on est all trop loin 
	movf	dividende, W		; on rappelle la derniere valeur du dividende negatif 
	addwf	diviseur, W			; qu'on rajoute au diviseur
	movwf	reste				; on copie le resultat dans le reste
	return						; et on quitte

; ********************************************************************************

binDec							; Routine principale
; division par 100 pour obtenir le chiffre des centaines	
	movwf	dividende			; on charge le dividende
	movlw	d'100'      		; on charge 100 
	movwf	diviseur			; dans le diviseur
	call	divOct				; on appelle la procdure division

	movf	resultat, W			; on rcupre le resultat de la division dans w		
	movwf	centaine			; on le stocke dans la variable 'cent'
	movf	reste, W    		; on rcupre le reste dans w

; division par 10 pour obtenir le chiffre des dizaines
	movwf	dividende			; on charge le reste dans le dividende
	movlw	D'10'				; on charge 10 
	movwf	diviseur			; dans le diviseur
	call	divOct				; on appelle la procdure division

	movf	resultat, W			; on rcupre le resultat de la division dans w
	movwf	dizaine				; on le stocke dans la variable 'diz'

; le reste est le chiffre des units
	movf	reste, W			; on rcupre le reste cd la valeur des units dans w
	movwf	unites				; on stocke dans variable 'unites'

; transformation en caractere ASCII (masque)
	movlw	B'00110000'			; masque correspondant aux entiers (0, 1, 2..) dans la 
								; table ASCII ex : le code ASCII de '0' 
								; est 48 = b'00110000' en binaire. 
								;     49 = b'00110001' etc...
	iorwf	centaine,f			; en faisant un 'o inclusif' du masque et de la 
								; valeur contenue dans la variable concerne on fait
								; simplement une addition 
	iorwf	dizaine,f			; idem pour les dizaines
	iorwf	unites,f			; idem pour les units
								; ce sont ces valeurs qui seront affiches sur le 
								; LCD
	return